@@ -1,5 +1,6 @@
 from . import exceptions
 import time
+from typing import Callable
 
 
 approval_exceptions = {
@@ -60,7 +61,7 @@ class MyAccess:
 
     def request_approval_by_name(self, profile_name: str, environment_name: str, application_name: str = None,
                                  justification: str = None, wait_time: int = 60, max_wait_time: int = 600,
-                                 block_until_disposition: bool = False) -> any:
+                                 block_until_disposition: bool = False, progress_func: Callable = None) -> any:
         """
         Requests approval to checkout a profile at a later time, using names of entities instead of IDs.
 
@@ -79,6 +80,7 @@ class MyAccess:
         :param block_until_disposition: Should this method wait/block until the request has been either approved,
             rejected, or withdrawn. If `True` then `wait_time` and `max_wait_time` will govern how long to wait before
             exiting.
+        :param progress_func: An optional callback that will be invoked as the checkout process progresses.
         :return: If `block_until_disposition = True` then returns the final status of the request. If
             `block_until_disposition = False` then returns details about the approval request.
         :raises ProfileApprovalMaxBlockTimeExceeded: if max_wait_time has been reached while waiting for approval.
@@ -92,11 +94,13 @@ class MyAccess:
             justification=justification,
             wait_time=wait_time,
             max_wait_time=max_wait_time,
-            block_until_disposition=block_until_disposition
+            block_until_disposition=block_until_disposition,
+            progress_func=progress_func
         )
 
     def request_approval(self, profile_id: str, environment_id: str, justification: str, wait_time: int = 60,
-                         max_wait_time: int = 600, block_until_disposition: bool = False) -> any:
+                         max_wait_time: int = 600, block_until_disposition: bool = False,
+                         progress_func: Callable = None) -> any:
         """
         Requests approval to checkout a profile at a later time, by supplying the names of entities vs. the IDs of
         those entities.
@@ -114,6 +118,7 @@ class MyAccess:
         :param block_until_disposition: Should this method wait/block until the request has been either approved,
             rejected, or withdrawn. If `True` then `wait_time` and `max_wait_time` will govern how long to wait before
             exiting.
+        :param progress_func: An optional callback that will be invoked as the checkout process progresses.
         :return: If `block_until_disposition = True` then returns the final status of the request. If
             `block_until_disposition = False` then returns details about the approval request.
         :raises ProfileApprovalMaxBlockTimeExceeded: if max_wait_time has been reached while waiting for approval.
@@ -137,6 +142,8 @@ class MyAccess:
                     if status == 'pending':
                         if time.time() >= quit_time:
                             raise exceptions.ProfileApprovalMaxBlockTimeExceeded()
+                        if progress_func:
+                            progress_func('awaiting approval')
                         time.sleep(wait_time)
                         continue
                     else:  # status == timeout or approved or rejected or cancelled
@@ -211,7 +218,7 @@ class MyAccess:
 
     def checkout(self, profile_id: str, environment_id: str, programmatic: bool = True,
                  include_credentials: bool = False, justification: str = None, wait_time: int = 60,
-                 max_wait_time: int = 600) -> dict:
+                 max_wait_time: int = 600, progress_func: Callable = None) -> dict:
         """
         Checkout a profile.
 
@@ -240,6 +247,7 @@ class MyAccess:
             was approved.
         :param max_wait_time: The maximum number of seconds to wait for an approval before throwing
             an exception.
+        :param progress_func: An optional callback that will be invoked as the checkout process progresses.
         :return: Details about the checked out profile, and optionally the credentials generated by the checkout.
         :raises ApprovalRequiredButNoJustificationProvided: if approval is required but no justification is provided.
         :raises ApprovalWorkflowTimedOut: if max_wait_time has been reached while waiting for approval.
@@ -261,8 +269,12 @@ class MyAccess:
         transaction = None
 
         # let's see if there is already a checked out profile
+        progress_pending_checked_out_profiles_sent = False
         while True:  # will break the loop when needed
             loop = False
+            if progress_func and not progress_pending_checked_out_profiles_sent:
+                progress_func('reviewing currently checked out profiles')
+                progress_pending_checked_out_profiles_sent = True
             for p in self.list_checked_out_profiles():
                 right_profile = p['papId'] == profile_id
                 right_env = p['environmentId'] == environment_id
@@ -274,6 +286,8 @@ class MyAccess:
                     else:  # we are in the middle of a profile being checked in so cannot check it out yet
                         loop = True
             if loop:
+                if progress_func:
+                    progress_func('pending profile checkin')
                 time.sleep(1)
             else:
                 break
@@ -302,7 +316,8 @@ class MyAccess:
                         justification=justification,
                         wait_time=wait_time,
                         max_wait_time=max_wait_time,
-                        block_until_disposition=True
+                        block_until_disposition=True,
+                        progress_func=progress_func
                     )
 
                     # handle the response based on the value of status
@@ -331,6 +346,8 @@ class MyAccess:
                 if transaction['status'] == 'checkOutInApproval':  # we have an approval workflow occurring
                     if time.time() >= quit_time:
                         raise exceptions.ApprovalWorkflowTimedOut()
+                    if progress_func:
+                        progress_func('awaiting approval')
                     time.sleep(wait_time)
                     continue
                 else:  # status == checkedOut
@@ -343,15 +360,18 @@ class MyAccess:
             credentials, transaction = self.credentials(
                 transaction_id=transaction_id,
                 transaction=transaction,
-                return_transaction_details=True
+                return_transaction_details=True,
+                progress_func=progress_func
             )
             transaction['credentials'] = credentials
 
+        if progress_func:
+            progress_func('complete')
         return transaction
 
     def checkout_by_name(self, profile_name: str, environment_name: str, application_name: str = None,
                          programmatic: bool = True, include_credentials: bool = False, justification: str = None,
-                         wait_time: int = 60, max_wait_time: int = 600) -> dict:
+                         wait_time: int = 60, max_wait_time: int = 600, progress_func: Callable = None) -> dict:
         """
         Checkout a profile by supplying the names of entities vs. the IDs of those entities.
 
@@ -380,6 +400,7 @@ class MyAccess:
             was approved.
         :param max_wait_time: The maximum number of seconds to wait for an approval before throwing
             an exception.
+        :param progress_func: An optional callback that will be invoked as the checkout process progresses.
         :return: Details about the checked out profile, and optionally the credentials generated by the checkout.
         :raises ApprovalRequiredButNoJustificationProvided: if approval is required but no justification is provided.
         :raises ApprovalWorkflowTimedOut: if max_wait_time has been reached while waiting for approval.
@@ -395,11 +416,12 @@ class MyAccess:
             include_credentials=include_credentials,
             justification=justification,
             wait_time=wait_time,
-            max_wait_time=max_wait_time
+            max_wait_time=max_wait_time,
+            progress_func=progress_func
         )
 
-    def credentials(self, transaction_id: str, transaction: dict = None,
-                    return_transaction_details: bool = False) -> any:
+    def credentials(self, transaction_id: str, transaction: dict = None, return_transaction_details: bool = False,
+                    progress_func: Callable = None) -> any:
         """
         Return credentials of a checked out profile given the transaction ID.
 
@@ -410,6 +432,7 @@ class MyAccess:
         :param transaction: Optional - the details of the transaction. Primary use is for internal purposes.
         :param return_transaction_details: Optional - whether to return the details of the transaction. Primary use is
             for internal purposes.
+        :param progress_func: An optional callback that will be invoked as the checkout process progresses.
         :return: Credentials associated with the checked out profile represented by the specified transaction.
         """
 
@@ -420,6 +443,8 @@ class MyAccess:
             while True:
                 transaction = self.get_checked_out_profile(transaction_id=transaction_id)
                 if transaction['status'] == 'checkOutSubmitted':  # async checkout process
+                    if progress_func:
+                        progress_func('credential creation')
                     time.sleep(1)
                     continue
                 else:  # status == checkedOut
